% 29.10.2019 - Multiple RISs and plain IOs (Methods 1-2-3)
% 96th Anniversary of Turkish Republic


% Method 3 included 09.10.2019
%
% ======================================================================
clear
%close all
clc
% basic inputs ==========================================================

fc=3000;     % MHz  Carrier frequency
F=32;        % sampling rate: fraction of wave length
V=10;        %  m/s MS1 speed 
NFFT=1024;    % Number of points in FFT
Nsamples=960 % Number of samples (may be increased up to 1000)

% geometry inputs ========================================================

BSx=-1000; % location of transmitter (BS) x-coordinate
BSy=0;  % location of transmitter (BS) y-coordinate
MS0=0;        % initial location of receiver (MS) x-coordinate

% locations of point scatterers =========================================

N=7;  % number of RISs (the first N SCs)
M=3;  % number of IOs  (remaining ones)


% SC=[rand(10,1)*500 + 250,  rand(10,1)*500];
 SC=[rand(N+M,1)*1000 + 250,  rand(N+M,1)*750];   % uniformly distributed 
 
%  SC =1.0e+03*[1.0288    0.3885
%     0.6735    0.7077
%     0.3408    0.4783
%     0.5165    0.7183
%     0.4037    0.1805
%     0.5310    0.5071
%     0.6901    0.2168
%     0.7771    0.5039
%     0.7074    0.5214
%     1.1254    0.0510];

SC =[802.2913  109.8862
  879.8834  141.8041
  281.9910   31.9893
  864.7134  476.3984
  612.4115  211.4001
  299.5326  403.9475
  739.5700  521.3723
  442.5104  374.3370
  373.0837  401.8508
  455.4942  333.8874];
 
%  % test 1
%  SC=[1000 0];
%  
%  % test 2
%   SC=[1000 0
%       500 tand(60)*500];

SCx=SC(:,1);
SCy=SC(:,2);

alpha=atand(SCy./SCx);  % ray angles for Doppler


NSC=length(SCx); % Number of scatterers; 

figure,plot(SCx,SCy,'*', BSx,BSy,'^',0,0,'square'), hold on
xlabel('x-axis');ylabel('y-axis')

% indirect parameters ===================================================
lambdac=300/fc;    % m wavelength
Dx=lambdac/F;      % m sampling spacing 
ts=Dx/V;          % s time sampling interval
fs=1/ts;           % Hz sampling frequency
kc=2*pi/lambdac;   % propagation constant

timeaxis=ts.*[0:Nsamples-1];

MSx=MS0+V.*timeaxis;  % MS route along x-axis
MSy=zeros(Nsamples);  % MS route along x-axis (y=0)
% plot(MSx,MSy,'r')
% xlabel('Distance (m)');
% ylabel('Distance (m)');

MINx=min(min(BSx, SCx))-100;
MAXx=max(max(BSx, SCx))+100;
MINy=min(min(min(BSy, SCy)))-100;
MAXy=max(max(max(BSy, SCy)))+900;
axis([MINx MAXx MINy MAXy])


% calculate distance matrix =============================================
distBSSC=sqrt((BSx-SCx).^2+(BSy-SCy).^2);

distBSSCext=repmat(distBSSC,1,Nsamples);

distSCMS=zeros(NSC,Nsamples);
for ii=1:Nsamples
    distSCMS(:,ii)=sqrt((SCx-MSx(ii)).^2+SCy.^2);
end

distBSSCMS=distBSSCext+distSCMS;

% calculate complex envelope ===========================================
ray=(lambdac./(4*pi*distBSSCMS(:,1))).*exp(-1i*kc*distBSSCMS);  % we assume the same distance in terms of LOS




R_IO=-1*ones(M,1);  % reflection coef. -1 for plain IOs

fD=V/lambdac;
Doppler=fD*cosd(alpha);  % Doppler shifts for all reflections

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% with errors (U=0 no errors)
U=0;
Doppler=Doppler + rand(10,1)*2*U-U;
% fD=fD+rand*2*U-U;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if N>0 
%%%%%%%%%%%%%%%%%%%%%%%%%%
% RIS Angles - Case 1 (ALIGN RISs TO THE LOS PATH)
theta_Case1=exp(1i*(-2*pi*Doppler(1:N)*timeaxis + kc*distBSSCMS(1:N,1) -2*pi*fD*timeaxis));


% RIS Angles - Cases 2 & 3 (ELIMINATE REFLECTIONS FROM PLAIN IOs)
if M>=N
theta2=zeros(N,N+M,Nsamples);

for i=1:N
    for k=N+1:N+M  
   
        % ith RIS eliminates kth IO
        theta2(i,k,:)=exp(1i*(-2*pi*Doppler(i)*timeaxis + kc*distBSSCMS(i,1) + 2*pi*Doppler(k)*timeaxis-kc*distBSSCMS(k,1) ));
         % all-zero terms inside theta2 are unused
    end
end

P=combinator(M,N,'p')+N; % Permutations without repetition (indices of IOs)
search=size(P,1);



theta_Case2=zeros(N,Nsamples); %  RIS angles
theta_Case3=zeros(N,Nsamples); %  RIS angles

sample=1; % time index
for ti=timeaxis(1:end)
Abs=zeros(1,search);

for ss=1:search
    theta=zeros(N,1); %  RIS angles for a specific time instant
    for n=1:N
   theta(n)=theta2(n,P(ss,n),sample) ;
    end
    
    r_ti= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*ti)  +  sum(theta.*ray(1:N,sample))+ sum(R_IO.*ray(N+1:end,sample));

    Abs(ss)=abs(r_ti);

end

[a1,a2_Case2]=max(Abs);  % find maximum max magnitude in all permutations

% This script ensures that variations are minimum in the received signal
if sample==1
   [a1,a2_Case3]=max(Abs);  % find maximum max magnitude in all permutations
   reference=a1;
else 
    [a1,a2_Case3]=min(abs(Abs-reference));
  %  [a1,a2_Case3]=min((Abs-reference).^2);
    reference=Abs(a2_Case3);
end


    for n=1:N
   theta_Case2(n,sample)=theta2(n,P(a2_Case2,n),sample) ;  % BEST RIS ANGLES for a time instant
   theta_Case3(n,sample)=theta2(n,P(a2_Case3,n),sample) ;  % BEST RIS ANGLES for a time instant
    end
    
  sample=sample +1;
end

    
elseif M<N && M>0
    % M RISs eliminate M plain IOs, remaining N-M RISs are aligned to the LOS
    % if M=0, Cases 2 & 3 are not possible
    
theta2=zeros(N,N+M,Nsamples);

for i=1:N
    for k=N+1:N+M  
   
        % ith RIS eliminates kth IO
        theta2(i,k,:)=exp(1i*(-2*pi*Doppler(i)*timeaxis + kc*distBSSCMS(i,1) + 2*pi*Doppler(k)*timeaxis-kc*distBSSCMS(k,1) ));
         % all-zero terms inside theta2 are unused
    end
end

P=combinator(N,M,'p'); % Permutations without repetition (indices of RISs)
search=size(P,1);

theta_Case2=zeros(N,Nsamples); %  RIS angles
theta_Case3=zeros(N,Nsamples); %  RIS angles

sample=1; % time index
for ti=timeaxis(1:end)
Abs=zeros(1,search);

for ss=1:search
    
    theta=zeros(N,1); %  RIS angles
    for n=1:M
   theta(P(ss,n))=theta2(P(ss,n),n+N,sample) ;  % eliminated IOs
    end
  
        % find the RISs that will be aligned to LOS
    ii=1;
    RIS_LOS=zeros(1,N-M);
    for counter=1:N
       if sum(double(counter==P(ss,:)))==0 
           RIS_LOS(ii)=counter;
           ii=ii+1;
       end
    end
    
    % RIS angles for aligning LOS
   theta(RIS_LOS)=exp(1i*(-2*pi*Doppler(RIS_LOS)*ti + kc*distBSSCMS(RIS_LOS,1) -2*pi*fD*ti)); 
    

    r_ti= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*ti)  +  sum(theta.*ray(1:N,sample))+ sum(R_IO.*ray(N+1:end,sample));
 
    
    Abs(ss)=abs(r_ti); 

end

% Determine the best permutation

[a1,a2_Case2]=max(Abs);  % find maximum max magnitude in all permutations

% This script ensures that variations are minimum in the received signal
if sample==1
   [a1,a2_Case3]=max(Abs);  % find maximum max magnitude in all permutations
   reference=a1;
else 
    [a1,a2_Case3]=min(abs(Abs-reference));
    reference=Abs(a2_Case3);
end


    for n=1:M
   theta_Case2(P(a2_Case2,n),sample)=theta2(P(a2_Case2,n),n+N,sample) ;  % eliminated IOs
   theta_Case3(P(a2_Case3,n),sample)=theta2(P(a2_Case3,n),n+N,sample) ;  % eliminated IOs
    end
  
        % find the RISs that will be aligned to LOS
    ii=1;
    RIS_LOS_Case2=zeros(1,N-M);
    for counter=1:N
       if sum(double(counter==P(a2_Case2,:)))==0 
           RIS_LOS_Case2(ii)=counter;
           ii=ii+1;
       end
    end
    
    
        ii=1;
    RIS_LOS_Case3=zeros(1,N-M);
    for counter=1:N
       if sum(double(counter==P(a2_Case3,:)))==0 
           RIS_LOS_Case3(ii)=counter;
           ii=ii+1;
       end
    end
    
    
    % RIS angles for aligning LOS
   theta_Case2(RIS_LOS_Case2,sample)=exp(1i*(-2*pi*Doppler(RIS_LOS_Case2)*ti + kc*distBSSCMS(RIS_LOS_Case2,1) -2*pi*fD*ti)); 
   theta_Case3(RIS_LOS_Case3,sample)=exp(1i*(-2*pi*Doppler(RIS_LOS_Case3)*ti + kc*distBSSCMS(RIS_LOS_Case3,1) -2*pi*fD*ti)); 
   
    

    
  sample=sample+1;  
    

end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%
end





if N==0 && M>1
% NO RIS 
    r= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +   sum(R_IO.*ray);
    
elseif N==0 && M==1
% NO RIS 
    r= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +   (R_IO.*ray);

elseif N>1 && M>1
% valid for both M>=N and N<M
r_Case1= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  sum(theta_Case1.*ray(1:N,:))+ sum(R_IO.*ray(N+1:end,:));
r_Case2= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  sum(theta_Case2.*ray(1:N,:))+ sum(R_IO.*ray(N+1:end,:));
r_Case3= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  sum(theta_Case3.*ray(1:N,:))+ sum(R_IO.*ray(N+1:end,:));

  
elseif N==1 && M>1
    
r_Case1= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  (theta_Case1.*ray(1,:))+ sum(R_IO.*ray(N+1:end,:));
r_Case2= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  (theta_Case2.*ray(1,:))+ sum(R_IO.*ray(N+1:end,:));
r_Case3= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  (theta_Case3.*ray(1,:))+ sum(R_IO.*ray(N+1:end,:));

elseif M==1 && N>1
    
r_Case1= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  sum(theta_Case1.*ray(1:N,:))+ (R_IO.*ray(end,:));
r_Case2= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  sum(theta_Case2.*ray(1:N,:))+ (R_IO.*ray(end,:));
r_Case3= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  sum(theta_Case3.*ray(1:N,:))+ (R_IO.*ray(end,:));

elseif N==1 && M==1

r_Case1= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  (theta_Case1.*ray(1,:))+ (R_IO.*ray(end,:));
r_Case2= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  (theta_Case2.*ray(1,:))+ (R_IO.*ray(end,:));
r_Case3= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  (theta_Case3.*ray(1,:))+ (R_IO.*ray(end,:));

elseif M==0 && N>1
% Case 2 is not possible

r_Case1= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  sum(theta_Case1.*ray(1:N,:));

elseif M==0 && N==1
% Case 2 is not possible
        
r_Case1= (lambdac./(4*pi*abs(BSx-MSx(1)))).*exp(-1i*2*pi*fD*timeaxis)  +  (theta_Case1.*ray(1,:));


end

% WE ASSUMED THAT RAYS ARE PARALLEL fD*cos(alpha) --> THE REASON OF SLIGHT DEVIATIONS IN PHASES AFTER 200 SAMPLES
% % unnoticable difference due to Doppler assumptions
% ray2=(lambdac./(4*pi*distBSSCMS(:,1))).*exp(1i*(2*pi*Doppler*timeaxis-kc*distBSSCMS(:,1)));
% plot(10*log10(abs(sum(ray))));hold on;plot(10*log10(abs(sum(ray2))))



if N>0 && M>0  % CASE 1 and CASE 2 and CASE 3
% plot amplitude and phase =============================================
figure,subplot(2,3,1)
plot(timeaxis,10*log10(abs(r_Case1)))
xlabel('Time (s)')
ylabel('Magnitude of complex envelope')
axis([0 ts*Nsamples -65 -40])
title('Method 1')

subplot(2,3,2)
 % plot amplitude and phase =============================================
plot(timeaxis,10*log10(abs(r_Case2)))
xlabel('Time (s)')
ylabel('Magnitude of complex envelope')
axis([0 ts*Nsamples -65 -40])
title('Method 2')

subplot(2,3,3)
 % plot amplitude and phase =============================================
plot(timeaxis,10*log10(abs(r_Case3)))
xlabel('Time (s)')
ylabel('Magnitude of complex envelope')
axis([0 ts*Nsamples -65 -40])
title('Method 3')


% plot normalized spectrum in dB =======================================
spectrumr=fftshift((abs(fft(r_Case1,NFFT))).^2);
freqaxis=[0:NFFT-1]*fs/NFFT-fs/2;
subplot(2,3,4),plot(freqaxis,10*log10(spectrumr)-max(10*log10(spectrumr)))
xlabel('Doppler shift (Hz)')
ylabel('Normalized frquency response (dB)')
 axis([-250 250 -60 0])
 title('Method 1')
 hold on
 

% plot normalized spectrum in dB =======================================
spectrumr=fftshift((abs(fft(r_Case2,NFFT))).^2);
freqaxis=[0:NFFT-1]*fs/NFFT-fs/2;
subplot(2,3,5),plot(freqaxis,10*log10(spectrumr)-max(10*log10(spectrumr)))
xlabel('Doppler shift (Hz)')
ylabel('Normalized frquency response (dB)')
 axis([-250 250 -60 0])
 title('Method 2')
 
 % plot normalized spectrum in dB =======================================
spectrumr=fftshift((abs(fft(r_Case3,NFFT))).^2);
freqaxis=[0:NFFT-1]*fs/NFFT-fs/2;
subplot(2,3,6),plot(freqaxis,10*log10(spectrumr)-max(10*log10(spectrumr)))
xlabel('Doppler shift (Hz)')
ylabel('Normalized frquency response (dB)')
 axis([-250 250 -60 0])
 title('Method 3')


10*log10(max(abs(r_Case1)))-10*log10(min(abs(r_Case1)))

10*log10(max(abs(r_Case2)))-10*log10(min(abs(r_Case2)))

10*log10(max(abs(r_Case3)))-10*log10(min(abs(r_Case3)))

mean(10*log10(abs(r_Case1)))
mean(10*log10(abs(r_Case2)))
mean(10*log10(abs(r_Case3)))


elseif N==0  % NO RIS, NO CASES
  figure,subplot(1,2,1)
plot(timeaxis,10*log10(abs(r)))
xlabel('Time (s)')
ylabel('Magnitude of complex envelope')
axis([0 ts*Nsamples -65 -40])

subplot(1,2,2)
spectrumr=fftshift((abs(fft(r,NFFT))).^2);
freqaxis=[0:NFFT-1]*fs/NFFT-fs/2;
plot(freqaxis,10*log10(spectrumr)-max(10*log10(spectrumr)))
xlabel('Doppler shift (Hz)')
ylabel('Normalized frquency response (dB)')
 axis([-250 250 -60 0])
 

elseif M==0 % ONLY CASE 1
    
    figure,subplot(1,2,1)
plot(timeaxis,10*log10(abs(r_Case1)))
xlabel('Time (s)')
ylabel('Magnitude of complex envelope')
axis([0 ts*Nsamples -65 -40])
title('Case 1')

% plot normalized spectrum in dB =======================================
spectrumr=fftshift((abs(fft(r_Case1,NFFT))).^2);
freqaxis=[0:NFFT-1]*fs/NFFT-fs/2;
subplot(1,2,2),plot(freqaxis,10*log10(spectrumr)-max(10*log10(spectrumr)))
xlabel('Doppler shift (Hz)')
ylabel('Normalized frquency response (dB)')
 axis([-250 250 -60 0])
 title('Case 1')
    
end
